# OTT mirror
# Copyright (C) 2014 Balthasar Szczepański
# bb2html.awk
# translate BBcode to HTML and add to post file.
# 27.10.2014

# This file is part of OTT mirror.
#
# OTT mirror is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affreo General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# OTT mirror is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affreo General Public License for more details.
# 
# You should have received a copy of the GNU Affreo General Public License
# along with OTT mirror. If not, see <http://www.gnu.org/licenses/>.

BEGIN{
	FS="="
	debug="debug=bb2html.awk:"
};
{
	eq=index($0,"=")
	argtab[substr($0,1,eq-1)]=substr($0,eq+1)
};
END{
	if(argtab["disable_bbcode"]!="")
	{
		debug=debug"###nl;BBcode disabled"
		print debug>>outfile
		exit 0
	}
	text=argtab["BB"]
	html=bb2html(text)
	print "HTML="html>>outfile
	print debug>>outfile
};

function bb2html(text,   depth,count,bbtree,html,tag,tagstart,taglength,elm,tagv,tagname,tagvalue,matchname)
{
	depth=0
	count[0]=1
	bbtree["0"]=""
	bbtree["0.n"]=0
	bbtree["0.t"]="post"
	bbtree["0.v"]=argtab["subject"]
	html=""
	gsub(/&#61;/,"=",text)
	debug = debug "###nl;###nl;Build BBcode tree:"
	while(length(text)!=0)
	{
		tag=match(text, /\[\/?[a-z0-9\*]+(=(([^\[<>\"\]]*)|(\"[^\"<>]*\")))?\]/)
		#"#\""
		# debug = debug "###nl;" RSTART " " RLENGTH
		if (tag==0)
		{
			debug = debug "###nl;text: " text
			bbtree[ind(count,depth)]=text
			bbtree[indt(count,depth)".n"]=count[depth]
			++count[depth]
			text=""
			break;
		}
		tagstart=RSTART
		taglength=RLENGTH
		if(tagstart>1)
		{
			elm=substr(text,1,tagstart-1)
			debug = debug "###nl;text: " elm
			bbtree[ind(count,depth)]=elm
			bbtree[indt(count,depth)".n"]=count[depth]
			++count[depth]
		}
		tag=substr(text,tagstart,taglength)
		text=substr(text,tagstart+taglength)
		debug = debug "###nl;tag: " tag
		if(tag!~/^\[\//)
		{
			tagv=index(tag,"=")
			if(tagv<2)
			{
				tagname=substr(tag,2,length(tag)-2)
				tagvalue=""
			}
			else
			{
				tagname=substr(tag,2,tagv-2)
				tagvalue=substr(tag,tagv+1,length(tag)-tagv-1)
			}
			if(tagname !~ /^((quote)|b|i|u|(code)|(img)|(url)|(size)|(color)|(center)|(right)|(sub)|(sup)|(s)|(spoiler)|(list)|(\*))$/)
			{
				debug = debug "###nl;unknown tag"
				bbtree[ind(count,depth)]=tag
				bbtree[indt(count,depth)".n"]=count[depth]
				++count[depth]
				continue
			}
			if((bbtree[indt(count,depth)".t"] == "code" && tagname!="code")||bbtree[indt(count,depth)".t"] == "img"||(bbtree[indt(count,depth)".t"] == "spoiler" && tagname=="spoiler")||(bbtree[indt(count,depth)".t"] == "size" && tagname=="size"))
			{
				debug = debug "###nl;this tag forbidden here"
				bbtree[ind(count,depth)]=tag
				bbtree[indt(count,depth)".n"]=count[depth]
				++count[depth]
				continue
			}
			if(tagname=="*")
			{
				if(bbtree[indt(count,depth)".t"]==tagname)
				{
					debug = debug "###nl;implied tag: [/"tagname"]"
					debug = debug "###nl;matched"
					bbtree[ind(count,depth)]="[/"tagname"]"
					bbtree[ind(count,depth)".k"]="e"
					bbtree[ind(count,depth)".m"]=1
					bbtree[ind(count,depth)".t"]=tagname
					bbtree[ind(count,depth)".v"]=bbtree[indt(count,depth)".v"]
					bbtree[indt(count,depth)".n"]=count[depth]
					bbtree[indt(count,depth)".m"]=1
					count[depth]=""
					--depth
					++count[depth]
				}
				else if(bbtree[indt(count,depth)".t"]!="list")
				{
					debug = debug "###nl;this tag forbidden here"
					bbtree[ind(count,depth)]=tag
					bbtree[indt(count,depth)".n"]=count[depth]
					++count[depth]
					continue
				}
			}
			debug = debug "###nl;name: " tagname " value: " tagvalue
			bbtree[ind(count,depth)]=tagname" "tagvalue
			bbtree[ind(count,depth)".k"]="s"
			bbtree[indt(count,depth)".n"]=count[depth]
			++depth
			count[depth]=1
			bbtree[indt(count,depth)]=tag
			bbtree[indt(count,depth)".n"]=0
			bbtree[indt(count,depth)".t"]=tagname
			bbtree[indt(count,depth)".v"]=tagvalue
		}
		else
		{
			if(depth<=0)
			{
				debug = debug "###nl;""unmatched"
				bbtree[ind(count,depth)]=tag
				bbtree[indt(count,depth)".n"]=count[depth]
				++count[depth]
				continue
			}
			tagname=substr(tag,3,length(tag)-3)
			if(tagname=="list"&&bbtree[indt(count,depth)".t"]=="*")
			{
				debug = debug "###nl;implied tag: [/*]"
				debug = debug "###nl;matched"
				bbtree[ind(count,depth)]="[/*]"
				bbtree[ind(count,depth)".k"]="e"
				bbtree[ind(count,depth)".m"]=1
				bbtree[ind(count,depth)".t"]="*"
				bbtree[ind(count,depth)".v"]=bbtree[indt(count,depth)".v"]
				bbtree[indt(count,depth)".n"]=count[depth]
				bbtree[indt(count,depth)".m"]=1
				count[depth]=""
				--depth
				++count[depth]
			}
			debug = debug "###nl;name: " tagname
			matchname=bbtree[indt(count,depth)".t"]
			if((bbtree[indt(count,depth)".t"] == "code" && tagname!="code")||(bbtree[indt(count,depth)".t"] == "img" && tagname != "img"))
			{
				debug = debug "###nl;this tag forbidden here"
				bbtree[ind(count,depth)]=tag
				bbtree[indt(count,depth)".n"]=count[depth]
				++count[depth]
				continue
			}
			if(tagname!=matchname)
			{
				debug = debug "###nl;mismatched: " matchname
				# bbtree[ind(count,depth)]=tag
				# bbtree[indt(count,depth)".n"]=count[depth]
				# ++count[depth]
				# continue
				bbtree[ind(count,depth)]=tag
				bbtree[ind(count,depth)".k"]="e"
				bbtree[ind(count,depth)".t"]=bbtree[indt(count,depth)".t"]
				bbtree[ind(count,depth)".v"]=bbtree[indt(count,depth)".v"]
				bbtree[indt(count,depth)".n"]=count[depth]
				count[depth]=""
				--depth
				++count[depth]
				continue
			}
			debug = debug "###nl;matched"
			bbtree[ind(count,depth)]=tag
			bbtree[ind(count,depth)".k"]="e"
			bbtree[ind(count,depth)".m"]=1
			bbtree[ind(count,depth)".t"]=tagname
			bbtree[ind(count,depth)".v"]=bbtree[indt(count,depth)".v"]
			bbtree[indt(count,depth)".n"]=count[depth]
			bbtree[indt(count,depth)".m"]=1
			count[depth]=""
			--depth
			++count[depth]
		}
	}
	if(depth>0)
		debug = debug "###nl;unclosed tags: " depth
	
	debug = debug "###nl;###nl;""generate HTML:"
	depth=0
	count[0]=0
	while(count[0]<=bbtree["0.n"]&&depth>=0)
	{
		if(count[depth]==0)
		{
			tag=bbtree[indt(count,depth)]
			tagname=bbtree[indt(count,depth)".t"]
			tagvalue=bbtree[indt(count,depth)".v"]
			debug = debug "###nl;name: "tagname" value: "tagvalue" elements: "bbtree[indt(count,depth)".n"]
			if(tagname == "post")
			{
			}
			else if(bbtree[indt(count,depth)".m"]=="")
			{
				debug = debug "###nl;unmatched"
				html = html tag
			}
			else if(tagname == "quote")
			{
				if (tagvalue=="")
				{
					html = html "<blockquote class=\"uncited\"><div>"
				}
				else
				{
					if(tagvalue !~ /^\".*\"$/)
					#"#\""
						tagvalue="You forgot about the quote marks, "tagvalue
					else
						tagvalue=bb2html(substr(tagvalue,2,length(tagvalue)-2))
					html = html "<blockquote><div><cite>"tagvalue" wrote:</cite>"
				}
			}
			else if (tagname == "b")
			{
				html = html"<span style=\"font-weight: bold\">"
				if(tagvalue!="")
					html=html"This tag shouldn't have any value. " 
			}
			else if (tagname == "i")
			{
				html = html"<span style=\"font-style: italic\">"
				if(tagvalue!="")
					html=html"This tag shouldn't have any value. " 
			}
			else if (tagname == "u")
			{
				html = html"<span style=\"text-decoration: underline\">"
				if(tagvalue!="")
					html=html"This tag shouldn't have any value. " 
			}
			else if (tagname == "code")
			{
				if(bbtree[indtu(count,depth)".t"] == "code")
				{
					html=html tag
					debug = debug "###nl;not a tag, actually"
				}
				else
				{
					html=html"<dl class=\"codebox\"><dt>Code: <a href=\"#\" onclick=\"selectCode(this); return false;\">Select all</a></dt><dd><code>"
					if(tagvalue!="")
						html=html"This tag shouldn't have any value. "
				}
			}
			else if (tagname == "img")
				html=html"<img src=\""
			else if (tagname == "url")
			{
				if(tagvalue=="")
				{
					++count[depth]
					tagvalue=bbtree[ind(count,depth)]
					--count[depth]
					debug = debug "###nl;implied value: " tagvalue
				}
				html=html"<a href=\""tagvalue"\" class=\"postlink\">"
			}
			else if (tagname == "size")
			{
				if (tagvalue==""||(tagvalue !~ /^[0-9]+$/)||(tagvalue+0)>200)
					html=html"<span style=\"font-size: 130%; line-height: 116%;\"> This tag should have a correct value. "
				else
					html=html"<span style=\"font-size: "tagvalue"%; line-height: 116%;\">"
			}
			else if (tagname == "color")
			{
				if (tagvalue==""||tagvalue !~ /^#[0-9A-F][0-9A-F][0-9A-F][0-9A-F][0-9A-F][0-9A-F]$/) #for some reason /^([0-9A-F]{6})$/ doesn't work.
					html=html"<span style=\"color: #EE0000\"> This tag should have a correct value. "
				else
					html=html"<span style=\"color: "tagvalue"\">"
			}
			else if (tagname ~ /^((center)|(right))$/)
			{
				html=html"<div align=\""tagname"\">"
				if(tagvalue!="")
					html=html"This tag shouldn't have any value. "
			}
			else if (tagname =="s")
			{
				html=html"<strike>"
				if(tagvalue!="")
					html=html"This tag shouldn't have any value. "
			}
			else if (tagname =="sub")
			{
				html=html"<sub>"
				if(tagvalue!="")
					html=html"This tag shouldn't have any value. "
			}
			else if (tagname =="sup")
			{
				html=html"<sup>"
				if(tagvalue!="")
					html=html"This tag shouldn't have any value. "
			}
			else if (tagname=="spoiler")
			{
				html=html"<div style=\"margin:20px; margin-top:5px\"><div class=\"quotetitle\"><b>Spoiler:</b> <input type=\"button\" value=\"Show\" style=\"width:45px;font-size:10px;margin:0px;padding:0px;\" onclick=\"if (this.parentNode.parentNode.getElementsByTagName('div')[1].getElementsByTagName('div')[0].style.display != '') { this.parentNode.parentNode.getElementsByTagName('div')[1].getElementsByTagName('div')[0].style.display = '';        this.innerText = ''; this.value = 'Hide'; } else { this.parentNode.parentNode.getElementsByTagName('div')[1].getElementsByTagName('div')[0].style.display = 'none'; this.innerText = ''; this.value = 'Show'; }\" /></div><div class=\"quotecontent\"><div style=\"display: none;\">"
				if(tagvalue!="")
					html=html"This tag shouldn't have any value. "
			}
			else if (tagname=="list")
			{
				if(tagvalue=="")
					html=html"<ul>"
				else
				{
					if (tagvalue=="1")
						tagvalue="decimal"
					else if (tagvalue=="i")
						tagvalue="lower-roman"
					else if (tagvalue=="I")
						tagvalue="upper-roman"
					else if (tagvalue=="a")
						tagvalue="lower-alpha"
					else if (tagvalue=="A")
						tagvalue="upper-alpha"
					else
					{
						html=html"This tag should have a correct value. "
						tagvalue="decimal"
					}
					html=html"<ol style=\"list-style-type: "tagvalue"\">"
				}
			}
			else if (tagname=="*")
			{
				html=html"<li>"
				if(tagvalue!="")
					html=html"This tag shouldn't have any value. "
			}
			else
			{
				debug = debug "###nl;unknown tag"
				html = html tag
			}
			++count[depth]
		}
		else if(bbtree[ind(count,depth)".k"]=="s")
		{
			debug = debug "###nl;entering tag:"
			++depth
			count[depth]=0
		}
		else if(bbtree[ind(count,depth)".k"]=="e")
		{
			debug = debug "###nl;leaving tag:"
			tag=bbtree[ind(count,depth)]
			tagname=bbtree[ind(count,depth)".t"]
			tagvalue=bbtree[ind(count,depth)".v"]
			debug = debug "###nl;name: "tagname" value: "tagvalue
			if (tagname == "post")
			{
			}
			else if(bbtree[ind(count,depth)".m"]=="")
			{
				debug = debug "###nl;unmatched"
				html = html tag
			}
			else if(tagname == "quote")
				html = html "</div></blockquote>"
			else if (tagname ~/^(b|i|u|(size)|(color))$/)
				html = html"</span>"
			else if (tagname == "code")
			{
				if(bbtree[indtu(count,depth)".t"] == "code")
				{
					html=html tag
					debug = debug "###nl;not a tag, actually"
				}
				else
					html = html"</code></dd></dl>"
			}
			else if (tagname == "img")
			{
				html = html"\" alt=\"Image\" />"
				if(tagvalue!="")
					html=html"This tag shouldn't have any value. "
			}
			else if(tagname=="url")
				html=html"</a>"
			else if (tagname ~ /^((center)|(right))$/)
				html=html"</div>"
			else if (tagname =="s")
				html=html"</strike>"
			else if (tagname =="sub")
				html=html"</sub>"
			else if (tagname =="sup")
				html=html"</sup>"
			else if (tagname=="spoiler")
				html=html"</div></div></div>"
			else if (tagname=="list")
			{
				if (tagvalue=="")
					html=html"</ul>"
				else
					html=html"</ol>"
			}
			else if (tagname=="*")
				html=html"</li>"
			else
			{
				debug = debug "###nl;unknown tag"
				html = html tag
			}
			count[depth]=""
			--depth
			++count[depth]
		}
		else if(count[depth]>bbtree[indt(count,depth)".n"])
		{
			debug = debug "###nl;run out of elements in this tag; leaving"
			count[depth]=""
			--depth
			++count[depth]
		}
		else
		{
			tag=bbtree[ind(count,depth)]
			debug = debug "###nl;text: "tag
			if(bbtree[indt(count,depth)".t"] == "code")
			{
				# gsub(/ ?<br> ?/,"<br>",tag)
				gsub(/ /,"\\&nbsp;",tag)
				gsub(/\t/,"\\&nbsp;\\&nbsp;",tag)
			}
			if(argtab["disable_magic_url"]==""&& bbtree[indt(count,depth)".t"] !~ /^((code)|(img)|(url))$/)
				gsub(/https?:\/\/[^\n\r\t<> ]+/,"<a href=\"&\" class=\"postlink\">&</a>",tag)
			if(argtab["disable_smilies"]=="" && bbtree[indt(count,depth)".t"] !~ /^((code)|(img))$/)
			{
				gsub(/(^|[\n\r\t ]):D($|[\n\r\t ])/," <img title=\"Very Happy\" alt=\":D\" src=\"/ott/prosilver/smilies/icon_biggrin.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):\)($|[\n\r\t ])/," <img title=\"Smile\" alt=\":)\" src=\"/ott/prosilver/smilies/icon_smile.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):\(($|[\n\r\t ])/," <img title=\"Sad\" alt=\":(\" src=\"/ott/prosilver/smilies/icon_sad.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):o($|[\n\r\t ])/," <img title=\"Surprised\" alt=\":o\" src=\"/ott/prosilver/smilies/icon_surprised.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):shock:($|[\n\r\t ])/," <img title=\"Shocked\" alt=\":shock:\" src=\"/ott/prosilver/smilies/icon_eek.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):\?($|[\n\r\t ])/," <img title=\"Confused\" alt=\":?\" src=\"/ott/prosilver/smilies/icon_confused.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ])8-\)($|[\n\r\t ])/," <img title=\"Cool\" alt=\"8-)\" src=\"/ott/prosilver/smilies/icon_cool.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):lol:($|[\n\r\t ])/," <img title=\"Laughing\" alt=\":lol:\" src=\"/ott/prosilver/smilies/icon_lol.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):x($|[\n\r\t ])/," <img title=\"Mad\" alt=\":x\" src=\"/ott/prosilver/smilies/icon_mad.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):P($|[\n\r\t ])/," <img title=\"Razz\" alt=\":P\" src=\"/ott/prosilver/smilies/icon_razz.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):oops:($|[\n\r\t ])/," <img title=\"Embarassed\" alt=\":oops:\" src=\"/ott/prosilver/smilies/icon_redface.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):cry:($|[\n\r\t ])/," <img title=\"Crying or Very Sad\" alt=\":cry:\" src=\"/ott/prosilver/smilies/icon_cry.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):evil:($|[\n\r\t ])/," <img title=\"Evil or Very Mad\" alt=\":evil:\" src=\"/ott/prosilver/smilies/icon_evil.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):twisted:($|[\n\r\t ])/," <img title=\"Twisted Evil\" alt=\":twisted:\" src=\"/ott/prosilver/smilies/icon_twisted.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):roll:($|[\n\r\t ])/," <img title=\"Rolling Eyes\" alt=\":roll:\" src=\"/ott/prosilver/smilies/icon_rolleyes.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):wink:($|[\n\r\t ])/," <img title=\"Wink\" alt=\":wink:\" src=\"/ott/prosilver/smilies/icon_wink.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]);\)($|[\n\r\t ])/," <img title=\"Wink\" alt=\":wink:\" src=\"/ott/prosilver/smilies/icon_wink.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):!:($|[\n\r\t ])/," <img title=\"Exclamation\" alt=\":!:\" src=\"/ott/prosilver/smilies/icon_exclaim.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):\?:($|[\n\r\t ])/," <img title=\"Question\" alt=\":?:\" src=\"/ott/prosilver/smilies/icon_question.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):idea:($|[\n\r\t ])/," <img title=\"Idea\" alt=\":idea:\" src=\"/ott/prosilver/smilies/icon_idea.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):arrow:($|[\n\r\t ])/," <img title=\"Arrow\" alt=\":arrow:\" src=\"/ott/prosilver/smilies/icon_arrow.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):\|($|[\n\r\t ])/," <img title=\"Neutral\" alt=\":|\" src=\"/ott/prosilver/smilies/icon_neutral.gif\" /> ",tag)
				gsub(/(^|[\n\r\t ]):mrgreen:($|[\n\r\t ])/," <img title=\"Mr. Green\" alt=\":mrgreen:\" src=\"/ott/prosilver/smilies/icon_mrgreen.gif\" /> ",tag)
			}
			
			html = html tag
			++count[depth]
		}
	}
	debug = debug "###nl;done"
	return html
}

function ind(count,depth,   ii,iindex)
{
	iindex = ""
	for(ii=0;ii<=depth;++ii)
	{
		if(ii!=0)
			iindex = iindex "."
		iindex = iindex count[ii]
	}
	# debug = debug "###nl;" iindex
	return iindex
}
function indt(count,depth,   ii,iindex)
{
	iindex = ""
	for(ii=0;ii<depth;++ii)
	{
		if(ii!=0)
			iindex = iindex "."
		iindex = iindex count[ii]
	}
	if(depth>0)
		iindex=iindex"."
	iindex = iindex "0"
	# debug = debug "###nl;" iindex
	return iindex
}
function indtu(count,depth,   ii,iindex)
{
	iindex = ""
	for(ii=0;ii<depth-1;++ii)
	{
		if(ii!=0)
			iindex = iindex "."
		iindex = iindex count[ii]
	}
	if(depth>0)
		iindex=iindex"."
	iindex = iindex "0"
	# debug = debug "###nl;" iindex
	return iindex
}